################################################################################
###
### AUTHOR:       Carlo Favero 
### DATE:         Fall 2023
### DESCRIPTION:  This script creates a database with all TS data 
### OUTPUT:       databases for the term  for the term
###               
###################################################################################

## -----------------------------------------------------------------------------
rm(list=ls())                                                                   # Clear the environment 

## -----------------------------------------------------------------------------
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))                     # Set the appropriate environment to work in every machine

## -----------------------------------------------------------------------------
listofpackages <- c("tidyverse","ellipse","reshape2","xts","xlsx","readxl",     # Install and load the required packages
"quantmod")   
source("../2024_ex1/preparePackages.R")
preparePackages(listofpackages)

## -----------------------------------------------------------------------------
# Nelson, Svensson, Siegel yield curve parametrization
nss <- function(n, params) {
  Beta0 <- params$BETA0
  Beta1 <- params$BETA1
  Beta2 <- params$BETA2
  Beta3 <- params$BETA3
  Tau1  <- params$TAU1
  Tau2  <- params$TAU2
  Tau1
  return(Beta0 + Beta1 * (1 - exp(-n / Tau1)) / (n / Tau1) +
           Beta2 * ((1 - exp(-n / Tau1)) / (n / Tau1) - exp(-n / Tau1)) +
           Beta3 * ((1 - exp(-n / Tau2)) / (n / Tau2) - exp(-n / Tau2)))
}

### YIELDS
### long-term yields from Gurkanyak 
### use the betas and the taus in the file to construct yields at all needed maturities
### name them in terms of quarters and add them to the database
### the file with all the info is yields_GURK.xlsx

yields<- read_excel('data/yields_GURK.xlsx', sheet=1, col_names=TRUE)
dates <-seq(as.Date("1971-12-01"),length=nrow(yields), by="quarters")

yields.ts <- xts(yields, order.by=dates)
yields.ts <- yields.ts[, -1]


# Take just the parameters
params.cols <- grep("^(BETA|TAU)", colnames(yields.ts))
params.ts <- yields.ts[, params.cols]
save(params.ts,file="data/yields_GURK_params.rda")
write.csv(data.frame(params.ts), "data/yields_GURK_params.csv", row.names = TRUE)

# Take the yields and save their names as "nX" where X is a number
yields.cols <- grep("^SVENY", colnames(yields.ts))
yields.ts <- yields.ts[, yields.cols]
colnames(yields.ts) <- gsub("^SVENY", "y", colnames(yields.ts))
new_colnames <- sapply(str_extract_all(colnames(yields.ts), "\\d+"), function(x) {
  paste0("y", as.integer(x) * 4)
})
colnames(yields.ts) <- new_colnames

# Since computations are a nightmare with time series, we convert everything to numeric
# and do everything with this class
Beta0 <- as.numeric(params.ts$BETA0)
Beta1 <- as.numeric(params.ts$BETA1)
Beta2 <- as.numeric(params.ts$BETA2)
Beta3 <- as.numeric(params.ts$BETA3)
Tau1  <- as.numeric(params.ts$TAU1)
Tau2  <- as.numeric(params.ts$TAU2)
N=60
ttm = c(1:N)*(1/4)
maturities<-c(1:N)
for (n in maturities){
  new_yield <-(Beta0 + Beta1 * (1 - exp(-0.25*n / Tau1)) / (0.25*n / Tau1) +
                 Beta2 * ((1 - exp(-0.25*n / Tau1)) / (0.25*n / Tau1) - exp(-0.25*n / Tau1)) +
                 Beta3 * ((1 - exp(-0.25*n / Tau2)) / (0.25*n / Tau2) - exp(-0.25*n / Tau2)))
  # Again to yields
  new_yield <- xts(new_yield, order.by=index(yields.ts))
  # Fancy header name
  colname   <- paste0("yNS", n)
  # Merge it with respect to the old dataset
  yields.ts <- merge(yields.ts, new_yield, join="left")
  colnames(yields.ts)[ncol(yields.ts)] <- colname
}
nss_yields<-0.01*yields.ts[,16:75]
colnames(nss_yields) <- paste("n",seq(1,N),sep = "_")
rm(list="params.ts","yields","yields.ts","new_yield")

save(nss_yields,file="data/yields1.rda")